package report

const (
	// pgStatDatabaseGeneralDescription is the detailed description of pg_stat_database (general stats) view
	pgStatDatabaseGeneralDescription = `Database-wide general statistics based on pg_stat_database view:

  column		origin		description
- datname		datname		Name of this database
- backends_total  	numbackends 	Total number of backends connected to the database
- commits		xact_commit	Number of transactions in this database that have been committed
- rollbacks		xact_rollback	Number of transactions in this database that have been rolled back
- read,KiB		blks_read	Number of data read in this database, in KiB
- hits			blks_hit	Number of times disk blocks were found already in the buffer cache, so that a read was 
					not necessary (this only includes hits in the PostgreSQL buffer cache, not the operating
					system's file system cache)
- returned		tup_returned	Number of rows returned by queries in this database
- fetched		tup_fetched	Number of rows fetched by queries in this database
- inserts		tup_inserted	Number of rows inserted by queries in this database
- updates		tup_updated	Number of rows updated by queries in this database
- deletes		tup_deleted	Number of rows deleted by queries in this database
- conflicts		conflicts	Number of queries canceled due to conflicts with recovery in this database.
- deadlocks		deadlocks	Number of deadlocks detected in this database
- csum_fails		checksum_failures	Number of data page checksum failures detected in this database
- temp_files		temp_files	Number of temporary files created by queries in this database.
- temp_bytes		temp_bytes	Total amount of data written to temporary files by queries in this database.
- read,ms		blk_read_time	Time spent reading data file blocks by backends in this database, in milliseconds
- write,ms		blk_write_time	Time spent writing data file blocks by backends in this database, in milliseconds
- stats_age		stats_reset	Age of collected statistics in the moment when stats are taken from this database

Details: https://www.postgresql.org/docs/current/static/monitoring-stats.html#PG-STAT-DATABASE-VIEW
`

	// pgStatDatabaseSessionsDescription is the detailed description of pg_stat_database (sessions stats) view
	pgStatDatabaseSessionsDescription = `Databases sessions statistics based on pg_stat_database view:

  column		origin				description
- datname		datname				Name of this database
- backends_total	numbackends			Total number of backends connected to the database
- session_total		session_time			Total time spent by database sessions in this database
- active_total		active_time			Total time spent executing SQL statements in this database
- idle_xact_total	idle_in_transaction_time	Total time spent idling while in a transaction in this database
- session,ms		session_time			Time spent by database sessions in this database, in milliseconds
- active,ms		active_time			Time spent executing SQL statements in this database, in milliseconds
- idle_xact,ms		idle_in_transaction_time	Time spent idling while in a transaction in this database, in milliseconds	
- sessions		sessions			Number of sessions established to this database
- abandoned		sessions_abandoned		Number of database sessions to this database that were terminated because connection to the client was lost
- fatal			sessions_fatal			Number of database sessions to this database that were terminated by fatal errors
- killed 		sessions_killed			Number of database sessions to this database that were terminated by operator intervention

Details: https://www.postgresql.org/docs/current/static/monitoring-stats.html#PG-STAT-DATABASE-VIEW
`

	// pgStatReplicationDescription is the detailed description of pg_stat_replication view
	pgStatReplicationDescription = `Streaming replication statistics based on pg_stat_replication view:

  column	origin			description
- pid		pid			Process ID of a WAL sender process
- client	client_addr		IP address of the client connected to this WAL sender
- user		usename			Name of the user logged into this WAL sender process
- name		application_name	Name of the application that is connected to this WAL sender
- state		state			Current WAL sender state
- mode		sync_state		Synchronous state of this standby server
- wal,KiB	-			Amount of WAL generated by server, in KiB
- pending,KiB	sent_lsn		Amount of WAL generated, but not sent to the standby, in KiB
- write,KiB	write_lsn		Amount of WAL sent, but not written by standby, in KiB
- flush,KiB	flush_lsn		Amount of WAL written by standby, but not flushed, in KiB
- replay,KiB	replay_lsn		Amount of WAL flushed by standby, but not replayed, in KiB
- total,KiB	replay_lsn		Total amount of WAL that have to be sent, written, flushed and replayed.
- write		write_lag		Time elapsed between flushing recent WAL locally and receiving notification that this standby
					server has written it (but not yet flushed it or applied it)
- flush		flush_lag		Time elapsed between flushing recent WAL locally and receiving notification that this standby
					server has written and flushed it (but not yet applied it)
- replay	replay_lag		Time elapsed between flushing recent WAL locally and receiving notification that this standby
					server has written, flushed and applied it
- horizon_xacts	backend_xmin		Number of transactions have to be replayed on this standby server
- horizon_age	-			Age of the xmin's horizon on this standby server

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-REPLICATION-VIEW
`

	// pgStatTablesDescription is the detailed description of pg_stat_all_tables and pg_statio_all_tables views
	pgStatTablesDescription = `Tables' statistics based on pg_stat_all_tables and pg_statio_all_tables views:

  column		origin			description
- relation		schemaname,relname	Name of the table, including schema
- seq_scan		seq_scan		Number of sequential scans initiated on this table
- seq_read		seq_tup_read		Number of live rows fetched by sequential scans
- idx_scan		idx_scan		Number of index scans initiated on this table
- idx_fetch		idx_tup_fetch		Number of live rows fetched by index scans
- inserts		n_tup_ins		Number of rows inserted
- updates		n_tup_upd		Number of rows updated (includes HOT updated rows)
- deletes		n_tup_del		Number of rows deleted
- hot_updates		n_tup_hot_upd		Number of rows HOT updated (i.e., with no separate index update required)
- live			n_live_tup		Estimated number of live rows
- dead			n_dead_tup		Estimated number of dead rows
- heap_read,KiB		heap_blks_read		Amount of data have been read from this table, in KiB
- heap_hit		heap_blks_hit		Number of buffer hits in this table
- idx_read,KiB		idx_blks_read		Amount of data have been read from all indexes on this table, in KiB
- idx_hit		idx_blks_hit		Number of buffer hits in all indexes on this table
- toast_read,KiB	toast_blks_read		Amount of data have been read from this table's TOAST table (if any), in KiB
- toast_hit		toast_blks_hit		Number of buffer hits in this table's TOAST table (if any)
- tidx_read,KiB		tidx_blks_read		Amount of data have been read from this table's TOAST table indexes (if any), in KiB
- tidx_hit		tidx_blks_hit		Number of buffer hits in this table's TOAST table indexes (if any)

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-ALL-TABLES-VIEW
         https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STATIO-ALL-TABLES-VIEW
`

	// pgStatIndexesDescription is the detailed description of pg_stat_all_indexes and pg_statio_all_indexes views
	pgStatIndexesDescription = `Indexes' statistics based on pg_stat_all_indexes and pg_statio_all_indexes views:

  column	origin				description
- index		schemaname,relname,indexrelname	Name of the index, including schema and table
- scan		idx_scan			Number of index scans initiated on this index
- tuples_read	idx_tup_read			Number of index entries returned by scans on this index
- tuples_fetch	idx_tup_fetch			Number of live table rows fetched by simple index scans using this index
- hit		idx_blks_hit			Number of buffer hits in this index
- read,KiB	idx_blks_read			Amount of data have been read from this index, in KiB

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-ALL-INDEXES-VIEW
         https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STATIO-ALL-INDEXES-VIEW
`

	// pgStatFunctionsDescription is the detailed description of pg_stat_user_functions view
	pgStatFunctionsDescription = `Functions' statistics based on pg_stat_user_functions view:

  column	origin			description
- funcid	funcid			OID of a function
- function	schemaname,funcname	Name of this function, including schema
- calls_total	calls			Total number of times this function has been called
- calls		calls			Number of times this function has been called per second
- total		total_time		Total time spent in this function and all other functions called by it
- self		self_time		Total time spent in this function itself, not including other functions called by it
- total_avg,ms	total_time,calls	Average time spent in this function and all other functions called by it, in milliseconds
- self_avg,ms	self_time,calls		Average time spent in this function itself, not including other functions called by it, in milliseconds

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-USER-FUNCTIONS-VIEW
`

	// pgStatWALDescription is the detailed description of pg_stat_wal view
	pgStatWALDescription = `WAL statistics based on pg_stat_wal view:

  column	origin			description
- source	-			Always has 'WAL' value
- waldir_size	-			Total size of WAL directory (pretty) 
- wal,KiB	wal_bytes		Amount of WAL generated, in KiB
- records	wal_records		Number of WAL records generated
- fpi		wal_fpi			Number of WAL full page images generated
- write		wal_write		Number of times WAL buffers were written out to disk via XLogWrite request
- sync		wal_sync		Number of times WAL files were synced to disk via issue_xlog_fsync request
- write,ms	wal_write_time		Amount of time spent writing WAL buffers to disk via XLogWrite request, in milliseconds 
- sync,ms	wal_sync_time		Amount of time spent syncing WAL files to disk via issue_xlog_fsync request, in milliseconds
- buffers_full	wal_buffers_full	Number of times WAL data was written to disk because WAL buffers became full
- stats_age	stats_reset		Age of collected statistics in the moment when stats are taken

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-WAL-VIEW
`

	// pgStatSizesDescription is the detailed description of stats about tables sizes
	pgStatSizesDescription = `Statistics about sizes of tables based on pg_*_size() functions:

  column		origin	description
- relation		-	Name of the table, including schema
- indexes_total		-	Total number of indexes on the table 
- size_total,KiB	-	Total size of the table, including metadata and indexes, in KiB
- table_total,KiB	-	Total size of the table, without metadata and indexes, in KiB
- meta_total,KiB	-	Total size of table metadata (VM, FSM, INIT forks), in KiB
- toast_total,KiB	-	Total size of table's TOASTed data, in KiB
- indexes_total,KiB	-	Total size of table's indexes, in KiB
- size,KiB		-	How does size of the table, including metadata and indexes is changing, in KiB
- table,KiB		-	How does size of the table, without metadata and indexes is changing, in KiB
- meta,KiB		-	How does size of the table's metadata is changing, in KiB
- toast,KiB		-	How does size of the table's TOASTed data is changing, in KiB
- indexes,KiB		-	How does size of the table's indexes is changing, in KiB

Details: https://www.postgresql.org/docs/current/functions-admin.html#FUNCTIONS-ADMIN-DBOBJECT
`

	// pgStatActivityDescription is the detailed description of pg_stat_activity view
	pgStatActivityDescription = `Activity statistics based on pg_stat_activity view:

  column	origin			description
- pid		pid			Process ID of this backend
- cl_addr	client_addr		IP address of the client connected to this backend
- cl_port	client_port		TCP port number that the client is using for communication with this backend
- datname	datname			Name of the database this backend is connected to
- usename	usename			Name of the user logged into this backend
- appname	application_name	Name of the application that is connected to this backend
- backend_type	backend_type		Type of current backend
- wait_etype	wait_event_type		The type of event for which the backend is waiting, if any
- wait_event	wait_event		Wait event name if backend is currently waiting
- state		state			Current overall state of this backend
- xact_age	xact_start		Current transaction's duration if active
- query_age	query_start		Current query's duration if active
- change_age	state_change		Age since last state has been changed
- query		query			Text of this backend's most recent query

Details: https://www.postgresql.org/docs/current/monitoring-stats.html#PG-STAT-ACTIVITY-VIEW
`

	// pgStatProgressVacuumDescription is the detailed description of pg_stat_progress_vacuum view
	pgStatProgressVacuumDescription = `Statistics about progress of vacuums based on pg_stat_progress_vacuum view:

  column		origin			description
- pid			pid			Process ID of this worker
- xact_age		xact_start		Current transaction's duration if active
- datname		datname			Name of the database this worker is connected to
- relation		relid			Name of the relation which is vacuumed by this worker
- state			state			Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- phase			phase			Current processing phase of vacuum
- size_total,KiB	heap_blks_total		Total size of the table, in KiB
- scanned_total,%	heap_blks_scanned	The percent of data scanned
- vacuumed_total,%	heap_blks_vacuumed	The percent of data vacuumed
- scanned,KiB		heap_blks_scanned	Amount of data scanned, in KiB
- vacuumed,KiB		heap_blks_vacuumed	Amount of data vacuumed, in KiB
- query			query			Text of this workers's "query"

Details: https://www.postgresql.org/docs/current/progress-reporting.html#VACUUM-PROGRESS-REPORTING
`

	// pgStatProgressClusterDescription is the detailed description of pg_stat_progress_cluster view
	pgStatProgressClusterDescription = `Statistics about progress of cluster and vacuum full operations based on pg_stat_progress_cluster view:

  column		origin			description
- pid			pid			Process ID of this worker
- xact_age		xact_start		Current transaction's duration if active
- datname		datname			Name of the database this worker is connected to
- relation		relid			Name of the relation which is processed by this worker
- index			cluster_index_relid	Name of the relation which is processed by this worker
- state			state			Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- phase			phase			Current processing phase of operation
- size_total,KiB	heap_blks_total		Total size of the table, in KiB
- scanned_total,%	heap_blks_scanned	The percent of data scanned
- tuples_scanned	heap_tuples_scanned	Number of heap tuples scanned
- tuples_written	heap_tuples_written	Number of heap tuples written
- query			query			Text of this workers's "query"

Details: https://www.postgresql.org/docs/current/progress-reporting.html#CLUSTER-PROGRESS-REPORTING
`

	// pgStatProgressCreateIndexDescription is the detailed description of pg_stat_progress_create_index view
	pgStatProgressCreateIndexDescription = `Statistics about progress of create index/reindex operations based on pg_stat_progress_create_index view:

  column		origin				description
- pid			pid				Process ID of this worker
- xact_age		xact_start			Current transaction's duration if active
- datname		datname				Name of the database this worker is connected to
- relation		relid				Name of the relation which is processed by this worker
- index			cluster_index_relid		Name of the relation which is processed by this worker
- state			state				Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- phase			phase				Current processing phase of operation
- locker_pid		current_locker_pid		Process ID of the locker currently being waited for
- lockers		lockers_total,lockers_done	Total number of lockers to wait for, and number of lockers already waited for.
- size_total/done,%	blocks_total,blocks_done	Total size to be processed and percent of already processed in the current phase, in KiB and percent
- tuples_total/done,%	tuples_total,tuples_done	Total number of tuples to be processed and percent of already processed in the current phase
- parts_total/done,%	partitions_total,partitions_done	Total number of partitions on which the index is to be created, and the number of partitions on which the index has been completed
- query			query				Text of this workers's "query"

Details: https://www.postgresql.org/docs/current/progress-reporting.html#CREATE-INDEX-PROGRESS-REPORTING
`

	// pgStatProgressAnalyzeDescription is the detailed description of pg_stat_progress_analyze view
	pgStatProgressAnalyzeDescription = `Statistics about progress of analyze operations based on pg_stat_progress_analyze view:

  column		origin				description
- pid			pid				Process ID of this worker
- xact_age		xact_start			Current transaction's duration if active
- datname		datname				Name of the database this worker is connected to
- relation		relid				Name of the relation which is processed by this worker
- state			state				Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- phase			phase				Current processing phase of operation
- sample_size,KiB	sample_blks_total		Total size of the sample, in KiB
- scanned,%		sample_blks_total,sample_blks_scanned	Total size of scanned sample, in percent
- ext_total/done	ext_stats_total,ext_stats_computed	Total number of extended statistics and number of already computed statistics
- child_total/done,%	child_tables_total,child_tables_done Total number of child tables and ratio of child tables already processed 
- child_in_progress	current_child_table_relid	Name of child relation which is processed by this worker

Details: https://www.postgresql.org/docs/current/progress-reporting.html#ANALYZE-PROGRESS-REPORTING
`

	// pgStatProgressBasebackupDescription is the detailed description of pg_stat_progress_basebackup view
	pgStatProgressBasebackupDescription = `Statistics about progress of basebackups performed based on pg_stat_progress_basebackup view:

  column		origin				description
- pid			pid				Process ID of this worker
- started_from  	client_addr			Network address of the client performed basebackup
- started_at		backend_start			Timestamp of when basebackup has been started
- duration		backend_start			Duration of basebackup
- state			state				Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- phase			phase				Current processing phase of operation
- size_total,KiB        	backup_total			Total amount of data that will be streamed, in KiB
- streamed,%		backup_total,backup_streamed	Total amount of data already streamed, in percent
- streamed,KiB			backup_total,backup_streamed	Amount of data streamed, in KiB
- tablespaces_total/streamed	tablespaces_total,tablespaces_streamed	Total number of tablespaces and already streamed.

Details: https://www.postgresql.org/docs/current/progress-reporting.html#BASEBACKUP-PROGRESS-REPORTING
`

	// pgStatProgressCopyDescription is the detailed description of pg_stat_progress_copy view
	pgStatProgressCopyDescription = `Statistics about progress of COPY operations performed, based on pg_stat_progress_copy view:

  column		origin				description
- pid			pid				Process ID of this worker
- xact_age		xact_start			Current transaction's duration if active
- datname		datname				Name of the database this worker is connected to
- relation		relid				Name of the relation which is processed by this worker
- state			state				Current overall state of this worker
- waiting		wait_event_type,wait_event	Wait event name and type for which the worker is waiting, if any
- command		command				The command that is running: COPY FROM, or COPY TO
- type			type				The io type that the data is read from or written to: FILE, PROGRAM, PIPE or CALLBACK
- size_total,KiB	-    				Size of the table, without metadata and indexes, in KiB
- source_total,KiB	bytes_total			Size of source file for COPY FROM command, in KiB. It is set to 0 if not available
- processed,KiB		bytes_processed			Amount of data already processed by COPY command, in KiB
- processed,%		bytes_total,bytes_processed	Ratio of processed data accordingly to sizeof source file, if available
- tuples_processed	tuples_processed		Number of tuples already processed by COPY command
- tuples_excluded	tuples_excluded			Number of tuples not processed because they were excluded by the WHERE clause of the COPY command

Details: https://www.postgresql.org/docs/current/progress-reporting.html#COPY-PROGRESS-REPORTING
`

	// pgStatStatementsTimingsDescription is the detailed description of pg_stat_statements section about timing stats
	pgStatStatementsTimingsDescription = `Statements timing statistics based on pg_stat_statements:

  column	origin			description
- user		rolname			Name of of user who executed the statement
- database	datname			Name of database in which the statement was executed
- all_total	total_time		Total time spent in the statement
- read_total	blk_read_time		Total time the statement spent reading blocks
- write_total	blk_write_time		Total time the statement spent writing blocks
- exec_total	-			Total time spent executing the statement (except time spent reading and writing blocks)
- all,ms	total_time		Time spent in the statement, in milliseconds
- read,ms	blk_read_time		Time the statement spent reading blocks, in milliseconds
- write,ms	blk_write_time		Time the statement spent writing blocks, in milliseconds
- exec,ms	-			Time spent in the statement, except time spent reading and writing blocks, in milliseconds
- calls		calls			Number of times executed
- queryid*	rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query		query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`

	// pgStatStatementsGeneralDescription is the detailed description of pg_stat_statements section about general stats
	pgStatStatementsGeneralDescription = `Statements general statistics based on pg_stat_statements:

  column	origin			description
- user		rolname			Name of of user who executed the statement
- database	datname			Name of database in which the statement was executed
- calls_total	calls			Total number of times executed
- rows_total	rows			Total number of rows retrieved or affected by the statement
- calls		calls			Number of times executed
- rows		rows			Number of rows retrieved or affected by the statement
- queryid	rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query		query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`

	// pgStatStatementsIODescription is the detailed description of pg_stat_statements section about buffers IO stats
	pgStatStatementsIODescription = `Statements statistics related to I/O, based on pg_stat_statements:

  column		origin			description
- user			rolname			Name of of user who executed the statement
- database		datname			Name of database in which the statement was executed
- hits_total		shared_blks_hit		Total number of shared block cache hits by the statement
- read_total,KiB	shared_blks_read	Total number of shared blocks read by the statement, in KiB
- dirtied_total,KiB	shared_blks_dirtied	Total number of shared blocks dirtied by the statement, in KiB
- written_total,KiB	shared_blks_written	Total number of shared blocks written by the statement, in KiB
- hits			shared_blks_hit		Number of shared block cache hits by the statement
- read,KiB		shared_blks_read	Number of shared blocks read by the statement, in KiB
- dirtied,KiB		shared_blks_dirtied	Number of shared blocks dirtied by the statement, in KiB
- written,KiB		shared_blks_written	Number of shared blocks written by the statement, in KiB
- calls			calls			Number of times executed
- queryid		rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query			query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`

	// pgStatStatementsTempDescription is the detailed description of pg_stat_statements section about temp files IO stats
	pgStatStatementsTempDescription = `Statements statistics related to temp I/O, based on pg_stat_statements:

  column		origin			description
- user			rolname			Name of of user who executed the statement
- database		datname			Name of database in which the statement was executed
- read_total,KiB	temp_blks_read		Total number of temp blocks read by the statement, in KiB
- write_total,KiB	temp_blks_written	Total number of temp blocks written by the statement, in KiB
- read,KiB		temp_blks_read		Number of temp blocks read by the statement, in KiB
- write,KiB		temp_blks_written	Number of temp blocks written by the statement, in KiB
- calls			calls			Number of times executed
- queryid		rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query			query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`

	// pgStatStatementsLocalDescription is the detailed description of pg_stat_statements section about local buffers IO stats
	pgStatStatementsLocalDescription = `Statements statistics related to local I/O, based on pg_stat_statements:

  column		origin			description
- user			rolname			Name of of user who executed the statement
- database		datname			Name of database in which the statement was executed
- hits_total		local_blks_hit		Total number of local block cache hits by the statement
- read_total,KiB	local_blks_read		Total number of local blocks read by the statement, in KiB
- dirtied_total,KiB	local_blks_dirtied	Total number of local blocks dirtied by the statement, in KiB
- written_total,KiB	local_blks_written	Total number of local blocks written by the statement, in KiB
- hits			local_blks_hit		Number of local block cache hits by the statement
- read,KiB		local_blks_read		Number of local blocks read by the statement, in KiB
- dirtied,KiB		local_blks_dirtied	Number of local blocks dirtied by the statement, in KiB
- written,KiB		local_blks_written	Number of local blocks written by the statement, in KiB
- calls			calls			Number of times executed
- queryid		rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query			query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`

	// pgStatStatementsWalDescription is the detailed description of pg_stat_statements section about WAL usage
	pgStatStatementsWalDescription = `Statements WAL usage statistics based on pg_stat_statements:

  column	origin			description
- user		rolname			Name of of user who executed the statement
- database	datname			Name of database in which the statement was executed
- wal_total,KiB	wal_bytes		Total amount of WAL bytes generated by the statement, in KiB
- wal,KiB	wal_bytes		Amount of WAL bytes generated by the statement, in KiB
- records	wal_records		Number of WAL records generated by the statement
- fpi		wal_fpi			Number of WAL full page images generated by the statement
- calls		calls			Number of times query executed
- queryid	rolname,datname,query	Fake queryid based on username, datname and text of the statement
- query		query			Text of a representative statement

Details: https://www.postgresql.org/docs/current/pgstatstatements.html
`
)
